@@ -1,19 +1,19 @@
 ;;; lang/javascript/mach.scm
-;;;
-;;; Copyright (C) 2015-2017 Matthew R. Wette
-;;;
-;;; This program is free software: you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by 
-;;; the Free Software Foundation, either version 3 of the License, or 
-;;; (at your option) any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+;; Copyright (C) 2015-2018 Matthew R. Wette
+;;
+;; This library is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU Lesser General Public
+;; License as published by the Free Software Foundation; either
+;; version 3 of the License, or (at your option) any later version.
+;;
+;; This library is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;; Lesser General Public License for more details.
+;;
+;; You should have received a copy of the GNU Lesser General Public License
+;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
 (define-module (nyacc lang javascript mach)
   #:export (js-spec
@@ -25,8 +25,13 @@
   #:use-module (nyacc parse)
   #:use-module (nyacc lex)
   #:use-module (nyacc util)
-  #:use-module ((srfi srfi-43) #:select (vector-map))
   )
+(cond-expand
+ (guile-2
+  (use-modules ((srfi srfi-43) #:select (vector-map))))
+ (else
+  (use-modules (nyacc compat18))))
+  
 
 ;; This parses EcmaScript v3 1999.  Some v5 2011 items are added as comments.
 
@@ -123,11 +128,14 @@
      )
 
     ;; from v5.1
-    #;(PropertyAssignment
+    #|
+    (PropertyAssignment
      (PropertyName ":" AssignmentExpression)
      ("get" PropertyName "(" ")" "{" FunctionBody "}")
-     ("set" PropertyName "(" PropertySetParametersList ")" "{" FunctionBody "}")
+     ("set" PropertyName "(" PropertySetParametersList ")"
+      "{" FunctionBody "}")
      )
+    |#
 
     (PropertyName
      (Identifier)
